# include <stdio.h>
# include <stdlib.h>
# include <time.h>
# include <math.h>
# include "ApEn.h"
# include "insn.h"
# include "nicelog.h"
//# define __DEBUG
//extern int result_buff[512];
int result_buff[APEN_MD_LEN+APEN_M1D_LEN] = {0};
//内存单元:用于计算的m维数组
int ApEn_MD_Buff[APEN_MD_LEN][APEN_DIMENSION_M];
//内存单元:用于计算的m+1维数组
int ApEn_M1D_Buff[APEN_M1D_LEN][APEN_DIMENSION_M1];
//内存单元:m维度的向量组的距离
float ApEn_DsitanceAll_m[APEN_MD_LEN];
//内存单元:m+1维度的向量组的距离
float ApEn_DsitanceAll_m1[APEN_M1D_LEN];

//变量：标准差 
float ApEn_StdVal = 0;
//变量: 标准差比例
float ApEn_StdProp = 0.2;
//变量: 阈值
float ApEn_Threshold = 0;
int ApEn_Threshold_int = 108;
//变量：m维距离对数平均值
float ApEn_DistLnAVR_m = 0; 
//变量：m+1维距离对数平均值
float ApEn_DistLnAVR_m1 = 0;
//变量：计算结果
float ApEn_Result = 0; 



//用户API接口函数
float ApEn_Calculate(float Prop, int* buff, int nicech)
{
	//初始化近似熵算法 
	ApEn_Algorithm_Init(Prop, buff); 
	//重构数组
	if(!nicech)
		ApEn_Restructure(buff, APEN_DATA_LEN);
	//计算各向量距离
	if(!nicech)
		ApEn_Calculate_All_Distance();
	else
		ApEn_Calculate_All_Distance_Nice((unsigned int)buff);
	//计算对数平均值
	if(!nicech)
		ApEn_Calculate_ln_AVR();
	else
		ApEn_Calculate_ln_AVR_Nice();
	//相减得出结果 
	ApEn_Sub_Result();
	//返回结果 
	return ApEn_Result;
}

//用户函数: 初始化算法
void ApEn_Algorithm_Init(float Prop, int* buff)
{
	//赋值比例 
	ApEn_StdProp = Prop;
	//计算标准差 
	ApEn_StdVal = ApEn_STD_Deviation(buff, APEN_DATA_LEN);	
	//计算阈值
	ApEn_Threshold = ApEn_StdVal * Prop;
	ApEn_Threshold_int = ApEn_Threshold;
}


//工具函数：计算标准差 
float ApEn_STD_Deviation(int* buff, unsigned int len)
{
	unsigned int i = 0;
	//总和 
	float sum = 0;
	//平均值
	float avr = 0;
	//临时变量
	float temp = 0;
	//标准差
	float std; 
	//计算数据总和 
	for(i = 0; i < len; ++i){
		sum += buff[i];		
	}
	//计算数据的平均值
	avr = sum / (float)len;
	//计算方差
	sum = 0;
	for(i = 0; i < len; ++i){
		//将每个元素减去平均值,并求平方 
		temp = 	(buff[i] - avr) * (buff[i] - avr); 	
		//计算总和
		sum += temp; 
	}
	//计算方差 
	avr = sum/len;
	//计算标准差 
	std = sqrt(avr);
	//返回标准差 
	return std;
}

//工具函数：重构数组
void ApEn_Restructure(int* buff, unsigned int len)
{
	unsigned int i = 0;
	unsigned int j = 0;
	//重构为m维度数组,一共有n-m+1个m维度向量 
	for(i = 0; i < (len - APEN_DIMENSION_M + 1); ++i){
		//每个向量为i,i+1,...,i+m-1
		//j从i开始,到i+m-1结束,一共取出m个数据 
		for(j = i; j < ((i + APEN_DIMENSION_M + 1) - 1); ++j){
			ApEn_MD_Buff[i][j-i] = buff[j];	
			
		}
	}
	//重构为m+1维度数组
	for(i = 0; i < (len - APEN_DIMENSION_M1 + 1); ++i){
		//每个向量为i,i+1,...,i+m-1
		//j从i开始,到i+m-1结束,一共取出m个数据 
		for(j = i; j < ((i + APEN_DIMENSION_M1 + 1) - 1); ++j){
			ApEn_M1D_Buff[i][j-i] = buff[j];				
		}	
	}	
}

//工具函数: 计算m维度向量距离(Vec为一维数组,m为维度)
int ApEn_Calculate_Distance(int* Vec1, int* Vec2, unsigned int m)
{
	unsigned int i = 0, j = 0;
	int element = 0, elementMax = 0;

	//计算每个对应元素的差值,并且取绝对值 
	for(i = 0; i < m; ++i){
			//求点的差 
			element = Vec1[i] - Vec2[i];	
			//求绝对值
			element = fabs(element);
			//如果比最大值大,就赋值给最大值 
			if(element > elementMax){
				elementMax = element;
			}
	}
	//返回的最大值即为距离 
	return elementMax;
}

//工具函数：使用nice协处理器计算各向量距离
void ApEn_Calculate_All_Distance_Nice(unsigned int buffAddr)
{
	int i = 0;
	custom_lbuf(buffAddr);
	custom_lth(ApEn_Threshold_int);
	custom_calu_apen((unsigned int)result_buff);
	for(i = 0; i < 249; ++i){
		//ApEn_DsitanceAll_m[i] = result_buff[i]*4211;//(result_buff[i]<<20)/APEN_MD_LEN;
		//ApEn_DsitanceAll_m[i] = custom_multiply_f(result_buff[i],MULT_PAR_M);
		ApEn_DsitanceAll_m[i] = custom_multiply_f(result_buff[i],0.00401606425702811244979919678715);
	}
	for(i = 249; i < 497; ++i){
		//ApEn_DsitanceAll_m1[i-249] = result_buff[i]*4228;//(result_buff[i]<<20)/APEN_M1D_LEN;
		//ApEn_DsitanceAll_m1[i-249] = custom_multiply_f(result_buff[i],MULT_PAR_M1);
		ApEn_DsitanceAll_m1[i-249] = custom_multiply_f(result_buff[i],0.00403225806451612903225806451613);
	}
}

//工具函数：计算各向量距离
void ApEn_Calculate_All_Distance(void)
{
	unsigned int i = 0, j = 0, Cnt = 0;
	int Distance = 0; 
	
	FILE* rfp;
	int num;
	# ifdef __DEBUG
	printf("=====================================2D=====================================\n");
	# endif
	//计算m维度向量的距离
	for(i = 0; i < APEN_MD_LEN; ++i){
		//计数器清零 
		Cnt = 0;
		for(j = 0; j < APEN_MD_LEN; ++j){
			//计算距离 
			Distance = ApEn_Calculate_Distance(ApEn_MD_Buff[i], ApEn_MD_Buff[j], APEN_DIMENSION_M);
			//如果距离小于等于设定的阈值
			if(Distance <= ApEn_Threshold){
				++Cnt;
			}
		}
		ApEn_DsitanceAll_m[i] = (float)Cnt/APEN_MD_LEN;
		# ifdef __DEBUG
			if(Cnt == result_buff[i]){
				printf("nice result %d: %d.pass.\n", i, result_buff[i]);
			}
			else{
				printf("nice result %d: %d. real result is %d. failed.\n", i, result_buff[i], Cnt);
			}
		# endif
	}	
	# ifdef __DEBUG
	printf("=====================================3D=====================================\n");
	# endif
	//计算m+1维度向量的距离
	for(i = 0; i < APEN_M1D_LEN; ++i){
		//计数器清零 
		Cnt = 0;
		for(j = 0; j < APEN_M1D_LEN; ++j){
			//计算距离 
			Distance = ApEn_Calculate_Distance(ApEn_M1D_Buff[i], ApEn_M1D_Buff[j], APEN_DIMENSION_M1);
			//如果距离小于等于设定的阈值
			if(Distance <= ApEn_Threshold){
				++Cnt;	
			}
		}
		ApEn_DsitanceAll_m1[i] = (float)Cnt/APEN_M1D_LEN;
		# ifdef __DEBUG
			if(Cnt == result_buff[i+249]){
				printf("nice result %d: %d.pass.\n", i+249, result_buff[i+249]);
			}
			else{
				printf("nice result %d: %d. real result is %d. failed.\n", i+249, result_buff[i+249], Cnt);
			}
		# endif
	}
}

//调用nice加速器计算log
float mylogf_nice(float x){
	float logx;
	int n = 0;
	float k = 1.0f;
	while(x < 1.0f){
		x = custom_multiply_f(x, 10.0f);
		--n;
	}
	while(x >= 10.0f){
		x = custom_multiply_f(x, 0.1f);
		++n;
	}
	logx = n;
	n = 30;
	while(n--){
		k = custom_multiply_f(k, 0.5f);
		if(custom_multiply_f(x,x) >= 10.0f){
			x = custom_multiply_f(custom_multiply_f(x,x), 0.1f);
			logx += k;
		}
		else{
			x = custom_multiply_f(x,x);
		}
	}
	return custom_multiply_f(logx, 2.30258509299);
}

//计算log
float mylogf(float x){
	float logx;
	int n = 0;
	float k = 1.0f;

	while(x < 1.0f){
		x *= 10;
		--n;
	}
	while(x >= 10.0f){
		x *= 0.1;
		++n;
	}
	logx = n;
	///for(i = 1; i <= 31; i++){
	n = 30;
	while(n--){
		k *= 0.5;
		if(x * x >= 10){
			x = x * x * 0.1;
			logx += k;
		}
		else{
			x=x*x;
		}
	}
	return (logx*2.30258509299);
	//return custom_multiply_f(logx, 2.30258509299);
}



//工具函数：对向量距离数组求对数平均值
void ApEn_Calculate_ln_AVR(void)
{
	unsigned int i = 0;
	float sum = 0;
	//对距离数组求对数平均值 
	for(i = 0; i < APEN_MD_LEN; ++i){
		sum += mylogf(ApEn_DsitanceAll_m[i]);
	}
	//计算平均值 
	ApEn_DistLnAVR_m = custom_multiply_f(sum, 0.004016064257f);
	//对距离数组求对数平均值 
	sum = 0;
	for(i = 0; i < APEN_M1D_LEN; ++i){
		sum += mylogf(ApEn_DsitanceAll_m1[i]);
	}
	//计算平均值 
	ApEn_DistLnAVR_m1 = sum / APEN_M1D_LEN;
}

//工具函数：对向量距离数组求对数平均值
void ApEn_Calculate_ln_AVR_Nice(void)
{
	unsigned int i = 0;
	float sum = 0;
	//对距离数组求对数平均值
	for(i = 0; i < APEN_MD_LEN; ++i){
		//求ln
		sum += mylogf_nice(ApEn_DsitanceAll_m[i]);
	}
	//计算平均值
	ApEn_DistLnAVR_m = custom_multiply_f(sum, 0.004016064257f);
	//ApEn_DistLnAVR_m = sum / APEN_MD_LEN;
	//对距离数组求对数平均值
	sum = 0;
	for(i = 0; i < APEN_M1D_LEN; ++i){
		//求ln
		sum += mylogf_nice(ApEn_DsitanceAll_m1[i]);
	}
	//计算平均值
	ApEn_DistLnAVR_m1 = custom_multiply_f(sum, 0.0040322580645f);
	//ApEn_DistLnAVR_m1 = sum / APEN_M1D_LEN;
}

//工具函数：算数平均值结果相减
void ApEn_Sub_Result(void)
{
	ApEn_Result = fabs(ApEn_DistLnAVR_m - ApEn_DistLnAVR_m1);
}














