# include "convolution.h"
# include "insn.h"
# include <stdio.h>
//============================================变量缓存定义============================================
//卷积计算器缓存
CALU_TYPE ConvCache0[MAX_COVN_CACHE_LEN];
CALU_TYPE ConvCache1[MAX_COVN_CACHE_LEN];
//卷积缓存位置指针
int ConvCachePos;
//卷积序列大小
int ConvSeqLen;
//卷积核缓存buff
CALU_TYPE ConvKernelCache[MAX_CONV_KERNEL_LEN];
//卷积核大小
int ConvKernelSize;
//卷积中间结果buff,其大小取决于卷积核的数量 
CALU_TYPE ConvMidBuff[PARALLEL_CALU_NUM];

//Padding数量
int ConvPaddingLen = 0;
//当前缓存位置选择 
int CacheCh = 0; 
//stride
int ConvStride = 3;


//============================================工具函数定义============================================
//设置Padding长度，padding方式: same:pad = 1,full: pad = ConvKernelSize - 1, valid:pad = 0
int Conv_SetPaddingLen(int PaddingLen)
{
	ConvPaddingLen = PaddingLen;
	return 0;
}

//设置Stride 
int Conv_SetStride(int stride)
{
	ConvStride = stride;
	return 0;
}

//加载原始数据序列
int Conv_LoadOriSeqToCache(CALU_TYPE*  OriDataBuff, int OriDataLen)
{
	int i = 0;
	int PaddingLen = 0;
	//缓存位置选择复位 
	CacheCh = 0;
	//计算padding长度 
	PaddingLen = ConvPaddingLen;
	//确认数据长度是否满足要求
	if((OriDataLen + (2*PaddingLen)) > MAX_COVN_CACHE_LEN){
		return ERROR_OVERFLOW;
	}
	//清零卷积缓存
	ConvCachePos = 0;
	//加载前置padding
	for(i = 0; i < PaddingLen; ++i){
		ConvCache0[ConvCachePos++] = 0;
	}
	//加载原始数据序列进入缓存
	for(i = 0; i < OriDataLen; ++i){
		ConvCache0[ConvCachePos++] = OriDataBuff[i];
	}
	//加载后置padding
	for(i = 0; i < PaddingLen; ++i){
		ConvCache0[ConvCachePos++] = 0;
	}
	//更新缓存数据长度 
	ConvSeqLen = ConvCachePos;
	//返回无错误
	return ERROR_NULL;
}

//加载卷积核 
int Conv_LoadKernel(CALU_TYPE*  KernelBuff, int KernelLen)
{
	int i = 0;
	//确认数据长度是否满足要求
	if(KernelLen > MAX_CONV_KERNEL_LEN){
		return ERROR_OVERFLOW;
	}	
	//加载Kernel 
	for(i = 0; i < KernelLen; ++i){
		ConvKernelCache[i] = KernelBuff[i];
	}
	//更新卷积核大小 
	ConvKernelSize = i;
	//返回无错误 
	return ERROR_NULL;
}

//调用Nice协处理器加载卷积核
int Conv_LoadKernel_Nice(CALU_TYPE*  KernelBuff, int KernelLen)
{
	int i = 0;
	//确认数据长度是否满足要求
	if(KernelLen > MAX_CONV_KERNEL_LEN){
		return ERROR_OVERFLOW;
	}
	//加载Kernel
	/*for(i = 0; i < KernelLen; ++i){
		ConvKernelCache[i] = KernelBuff[i];
	}*/
	custom_cnn_load_kernel((unsigned int)KernelBuff, KernelLen);
	//更新卷积核大小
	ConvKernelSize = KernelLen;
	//返回无错误
	return ERROR_NULL;
}

//设置原始序列长度 
int Conv_SetSeqLen(int len)
{
	if(len <= MAX_COVN_CACHE_LEN)
		ConvSeqLen = len;
	else
		return ERROR_OVERFLOW;
	return ERROR_NULL;
}

//清零缓存
void Conv_ClearConvCache(void)
{
	//缓存位置指针清零 
	ConvCachePos = 0;
}

//卷积向量计算函数 
CALU_TYPE Conv_CaluFunction(CALU_TYPE* StartAddr)
{
	int i = 0;
	CALU_TYPE sum = 0;
	//进行一次向量乘法 
	for(i = 0; i < ConvKernelSize; ++i){
		//sum += 	StartAddr[i] * ConvKernelCache[ConvKernelSize-i-1];
		//改成顺序模式 
		sum += 	StartAddr[i] * ConvKernelCache[i];
	}
	return sum;
}

//进行卷积,CacheCh选择缓存, StartPoint缓存数据起始地址 
int Conv_RunningConvolution(int StartPoint)
{
	int i = 0, j = 0,k = 0;
	int PaddingLen = 0;
	//计算padding长度 
	PaddingLen = ConvPaddingLen;
	//===================先padding=================== 
	for(i = 0; i < PaddingLen; ++i){
		//padding
		if(!CacheCh) 
			ConvCache1[ConvCachePos++] = 0;
		else
			ConvCache0[ConvCachePos++] = 0;
		//如果超出了缓存最大长度，返回溢出错误 
		if(ConvCachePos >= MAX_COVN_CACHE_LEN){
			//更新卷积序列大小 
			ConvSeqLen = ConvCachePos; 
			return ERROR_OVERFLOW;
		}
	}
	//===============进行卷积计算=============== 
	for(i = 0; i <= ConvSeqLen-ConvKernelSize; i += ConvStride){
		//每步卷积计算后，将结果存入原来的缓存中
		if(!CacheCh)
			ConvCache1[ConvCachePos++] = Conv_CaluFunction((CALU_TYPE*)ConvCache0+i+j+StartPoint);
		else
			ConvCache0[ConvCachePos++] = Conv_CaluFunction((CALU_TYPE*)ConvCache1+i+j+StartPoint);
		//如果超出了缓存最大长度，返回溢出错误
		if(ConvCachePos >= MAX_COVN_CACHE_LEN){
			//更新卷积序列大小
			ConvSeqLen = ConvCachePos;
			return ERROR_OVERFLOW;
		}
	}
	//===================后padding===================
	for(i = 0; i < PaddingLen; ++i){
		//padding
		if(!CacheCh) 
			ConvCache1[ConvCachePos++] = 0;
		else
			ConvCache0[ConvCachePos++] = 0;
		//如果超出了缓存最大长度，返回溢出错误 
		if(ConvCachePos >= MAX_COVN_CACHE_LEN){
			//更新卷积序列大小 
			ConvSeqLen = ConvCachePos; 
			return ERROR_OVERFLOW;
		}
	}
	return ERROR_NULL;
}


//调用Nice协处理器进行卷积,CacheCh选择缓存, StartPoint缓存数据起始地址
int Conv_RunningConvolution_Nice(int StartPoint)
{
	int i = 0, j = 0,k = 0;
	int PaddingLen = 0;
	//计算padding长度
	PaddingLen = ConvPaddingLen;
	//===================先padding===================
	for(i = 0; i < PaddingLen; ++i){
		//padding
		if(!CacheCh)
			ConvCache1[ConvCachePos++] = 0;
		else
			ConvCache0[ConvCachePos++] = 0;
		//如果超出了缓存最大长度，返回溢出错误
		if(ConvCachePos >= MAX_COVN_CACHE_LEN){
			//更新卷积序列大小
			ConvSeqLen = ConvCachePos;
			return ERROR_OVERFLOW;
		}
	}
	//===============进行卷积计算===============
	for(i = 0; i <= ConvSeqLen-ConvKernelSize; i += j){
		//这里本应该是并行计算，在此程序中使用循环代替，进行PARALLEL_CALU_NUM次并行计算
		for(j = 0; j < PARALLEL_CALU_NUM * ConvStride;){
			//每步卷积计算后，将结果存入原来的缓存中
			if(!CacheCh)
				//ConvCache1[ConvCachePos++] = Conv_CaluFunction((CALU_TYPE*)ConvCache0+i+j+StartPoint);
				ConvCache1[ConvCachePos++] = custom_cnn_calculate((unsigned int)(ConvCache0+i+j+StartPoint));
				else
				//ConvCache0[ConvCachePos++] = Conv_CaluFunction((CALU_TYPE*)ConvCache1+i+j+StartPoint);
				ConvCache0[ConvCachePos++] = custom_cnn_calculate((unsigned int)(ConvCache1+i+j+StartPoint));
			//j步进stride
			j = j + ConvStride;
			//如果已经卷积完成则退出循环
			if(i + j  >= ConvSeqLen-ConvKernelSize+1){
				break;
			}
			//如果超出了缓存最大长度，返回溢出错误
			if(ConvCachePos >= MAX_COVN_CACHE_LEN){
				//更新卷积序列大小
				ConvSeqLen = ConvCachePos;
				return ERROR_OVERFLOW;
			}
		}
	}
	//===================后padding===================
	for(i = 0; i < PaddingLen; ++i){
		//padding
		if(!CacheCh)
			ConvCache1[ConvCachePos++] = 0;
		else
			ConvCache0[ConvCachePos++] = 0;
		//如果超出了缓存最大长度，返回溢出错误
		if(ConvCachePos >= MAX_COVN_CACHE_LEN){
			//更新卷积序列大小
			ConvSeqLen = ConvCachePos;
			return ERROR_OVERFLOW;
		}
	}
	return ERROR_NULL;
}

//最大函数 
CALU_TYPE Tools_GetMax(CALU_TYPE a, int clear)
{
	static CALU_TYPE maxval = 0;
	//clear 1代表清零 
	if(clear){
		maxval = 0;
	}
	else if(a > maxval){
		maxval = a;
	}
	return maxval;
}

//平均函数 
CALU_TYPE Tools_GetAvr(CALU_TYPE a, int clear)
{
	static CALU_TYPE avrval = 0;
	static int count = 0;
	static CALU_TYPE sum = 0;
	//clear 1代表清零 
	if(clear){
		avrval = 0;
		count = 0;
		sum = 0;
	}
	else{
		++count;
		sum = sum + a;
		avrval = sum/count;
	}
	return avrval;
}

//进行池化
int Conv_RunningPooling(int PoolingMode, int PoolingLen)
{
	//Posa用于当前使用的cache,Posb用于Pooling后写入数据的cache 
	int Posa = 0, Posb = 0,i = 0,j = 0,run_num = 0; 
	CALU_TYPE tempval; 
	//根据卷积缓存位置和卷积序列的长度计算需要执行多少次 
	run_num = ConvCachePos/ConvSeqLen;
	//依次对每个卷积序列进行操作 
	for(i = 0; i < run_num; ++i){
		//前padding补0 
		for(j = 0; j < ConvPaddingLen; ++j){
			if(CacheCh == 0)
				ConvCache1[Posb] = 0;
			else
				ConvCache0[Posb] = 0;
			//两个位置都+1 
			++Posa;
			++Posb;
		}
		//清零工具函数 
		Tools_GetMax(0,1);
		Tools_GetAvr(0,1);
		//对卷积序列本体进行pooling操作
		for(j = 0; j < ConvSeqLen - (2*ConvPaddingLen); ++j){
			if(PoolingMode == POOLING_MODE_MAX){
				//计算最大值
				if(CacheCh == 0){
					tempval = Tools_GetMax(ConvCache0[Posa+j],0);
				}
				else{
					tempval = Tools_GetMax(ConvCache1[Posa+j],0);			
				}
			}
			else{
				//计算平均值
			 	if(CacheCh == 0)
					tempval = Tools_GetAvr(ConvCache0[Posa+j],0);
				else
					tempval = Tools_GetAvr(ConvCache1[Posa+j],0);
			}
			//如果j+1能被PoolingLen整除，则需要load进缓存 
			if((j + 1) % PoolingLen == 0){
				if(CacheCh == 0)
					ConvCache1[Posb++] = tempval;
				else
					ConvCache0[Posb++] = tempval;
				//清零工具函数 
				Tools_GetMax(0,1);
				Tools_GetAvr(0,1);
			}
		}
		//Posa循环执行完成后需要+ConvSeqLen
		Posa += ConvSeqLen - (2*ConvPaddingLen);
		//后padding补0 
		for(j = 0; j < ConvPaddingLen; ++j){
			if(CacheCh == 0)
				ConvCache1[Posb] = 0;
			else
				ConvCache0[Posb] = 0;
			//两个位置都+1 
			++Posa;
			++Posb;
		}
	}
	//更新缓存计数器，以及卷积序列大小 
	ConvCachePos = Posb;
	ConvSeqLen = ConvCachePos/run_num;
	return ERROR_NULL;
}

//结束当前层的运算
void Conv_LayerEnd(void)
{
	if(CacheCh)	CacheCh = 0;
	else		CacheCh = 1;
}
