# ifndef _CNN_CONVVOLUTION_H__
# define _CNN_CONVVOLUTION_H__

//============================================程序参数配置============================================
//卷积缓存最大长度(此参数规定了程序的卷积序列缓存大小，若计算结果的数据量高于这个值，则程序无法运行)
# define MAX_COVN_CACHE_LEN		15000
//卷积核最大长度(在实际IC中，此参数决定了NPU每个核心乘法器的使用数量)
# define MAX_CONV_KERNEL_LEN	31
//并行计算数量(此参数规定了程序并行计算的数量，在转为实际电路后，此参数则是NPU的核心数量)
# define PARALLEL_CALU_NUM		8

//计算方式:int or float，在此手动更改
# define CALU_TYPE				float
//错误码定义：
# define ERROR_NULL				0
# define ERROR_OVERFLOW			1
//池化方式的定义
# define POOLING_MODE_MAX		0
# define POOLING_MODE_AVR		1 


//============================================变量缓存声明============================================
//==================================这里的变量不允许用户自行写入数据==================================
//卷积计算器缓存（乒乓机制） 
extern CALU_TYPE ConvCache0[MAX_COVN_CACHE_LEN];
extern CALU_TYPE ConvCache1[MAX_COVN_CACHE_LEN];
//卷积缓存位置指针
extern int ConvCachePos;
//卷积序列大小
extern int ConvSeqLen;
//卷积核缓存buff
extern CALU_TYPE ConvKernelCache[MAX_CONV_KERNEL_LEN];
//卷积核大小
extern int ConvKernelSize;
//卷积中间结果buff,其大小取决于卷积核的数量 
//extern CALU_TYPE ConvMidBuff[PARALLEL_CALU_NUM];
//Padding数量
extern int ConvPaddingLen;
//stride
extern int ConvStride;

//============================================工具函数声明============================================
//设置Padding长度，padding方式: same:pad = 1,full: pad = ConvKernelSize - 1, valid:pad = 0
int Conv_SetPaddingLen(int PaddingLen);
//设置Stride 
int Conv_SetStride(int stride);
//设置原始序列长度 
int Conv_SetSeqLen(int len);
//加载原始数据序列
int Conv_LoadOriSeqToCache(CALU_TYPE*  OriDataBuff, int OriDataLen);
//加载卷积核 
int Conv_LoadKernel(CALU_TYPE*  KernelBuff, int KernelLen); 
//加载卷积核
int Conv_LoadKernel_Nice(CALU_TYPE*  KernelBuff, int KernelLen);
//清零缓存
void Conv_ClearConvCache(void);
//进行卷积
int Conv_RunningConvolution(int StartPoint);
//调用Nice协处理器进行卷积,StartPoint缓存数据起始地址
int Conv_RunningConvolution_Nice(int StartPoint);
//进行池化
int Conv_RunningPooling(int PoolingMode, int PoolingLen);
//结束当前层的运算
void Conv_LayerEnd(void); 





# endif
