// See LICENSE for license details.
#include <stdio.h>
#include <time.h>
#include <stdlib.h>
#include "hbird_sdk_soc.h"
#include "math.h"
#include "insn.h"
#include "ApEn.h"
#include "cnn_top.h"
#include "hbirdv2_uart.h"
#include "hbirdv2_gpio.h"

//原始计算数据
extern const float FullNetPara0[128][80];
extern const float FullNetPara1[32][128];
extern const float FullNetPara2[6][32];
extern const float kernel0[11];
extern float OriDataBuff[250];
extern int Apenbuff[250];
//全连接层的计算结果
float ResultBuff[6];
int ClassBuff[6] = {0,0,0,0,0,0};
//实例化CNN计算库
CNN_NET cnninst1;
//初始化CNN,配置CNN程序的参数，通过配置这个函数可以建立网络关系
void Init_CNN(void);
//卷积神经网络计算实现函数
void LoadOriData_Calu(int usenice);
//用于对照的卷积向量计算函数
float MyConv_CaluFunction(float* buff1, float* buff2, int len);
//用于浮点卷积计算器的测试数据
float buffconv1[17] = {1.1,1.2,1.3,1.4,1.5,1.6,1.7,1.8,1.9,2.0,2.1,2.2,2.3,2.4,2.5,2.6,2.7};
float buffconv2[17] = {1.6,1.7,1.8,1.9,2.0,2.1,2.2,2.3,2.5,2.6,2.7,2.8,2.9,3.0,3.1,3.1,3.2};

//主函数
int main(void)
{
	int i,j;
	int rt1;
	unsigned int begin_instret, end_instret, instret_normal, instret_nice;
	unsigned int begin_cycle,   end_cycle,   cycle_normal,   cycle_nice;
	float conv_result;
	//=============================操作提示=====================================
	//===如果需要调用NICE处理器计算CNN,需要在cnn_top.h中定义# define CNN_NICE_USE
	//===如果不需要调用NICE处理器计算CNN,则取消定义# define CNN_NICE_USE=========
	printf("***********************Tips:************************\n");
	printf("*Test for Physiological Signal Processor.\n");
	printf("*Author: XiaPeng LiXianLong ZhangTao.\n");
	printf("*Date: 2022.05.19.\n\n\n");
	//=============================卷积计算正确性测试====================================
	printf("=================Start Convolution Correctness Test!=================\n");
	//卷积计算5
	conv_result = MyConv_CaluFunction(buffconv1, buffconv2, 5);
	printf("conv_result real = %f\n", conv_result);
	rt1 = custom_cnn_load_kernel((unsigned int)buffconv2, 5);
	conv_result = custom_cnn_calculate((unsigned int )buffconv1);
	printf("conv_result nice = %f\n", conv_result);
	//卷积计算9
	conv_result = MyConv_CaluFunction(buffconv1, buffconv2, 9);
	printf("conv_result real = %f\n", conv_result);
	rt1 = custom_cnn_load_kernel((unsigned int)buffconv2, 9);
	conv_result = custom_cnn_calculate((unsigned int )buffconv1);
	printf("conv_result nice = %f\n", conv_result);

	//卷积计算15
	conv_result = MyConv_CaluFunction(buffconv1, buffconv2, 15);
	printf("conv_result real = %f\n", conv_result);
	rt1 = custom_cnn_load_kernel((unsigned int)buffconv2, 15);
	conv_result = custom_cnn_calculate((unsigned int )buffconv1);
	printf("conv_result nice = %f\n", conv_result);
	//=============================卷积神经网络计算测试=============================
# ifdef CNN_NICE_USE
	printf("=========================Start NICE CNN Test!========================\n");
	begin_instret =  __get_rv_instret();
	begin_cycle   =  __get_rv_cycle();
	//test
	Init_CNN();																	//初始化CNN
	LoadOriData_Calu(1);														//计算CNN
	//end test
	end_instret = __get_rv_instret();
	end_cycle   = __get_rv_cycle();
	instret_nice = end_instret - begin_instret;
	cycle_nice = end_cycle - begin_cycle;
	printf("#instret: %d, cycle: %d \n", instret_nice, cycle_nice);
	//输出分类结果
	printf("@Result:\n");
	for(i = 0; i < 6; ++i){
		printf("class %d : %d ", i, ClassBuff[i]);
		printf("%f \n", ResultBuff[i]);
	}
# endif
	for(i = 0; i < 6; ++i){
		ClassBuff[i] = 0;
	}
	printf("=========================Start Soft CNN Test!========================\n");
	begin_instret =  __get_rv_instret();
	begin_cycle   =  __get_rv_cycle();
	//test
	Init_CNN();																	//初始化CNN
	LoadOriData_Calu(0);														//计算CNN
	//end test
	end_instret = __get_rv_instret();
	end_cycle   = __get_rv_cycle();
	instret_nice = end_instret - begin_instret;
	cycle_nice = end_cycle - begin_cycle;
	printf("#instret: %d, cycle: %d \n", instret_nice, cycle_nice);
	//输出分类结果
	printf("@Result:\n");
	for(i = 0; i < 6; ++i){
		printf("class %d : %d ", i, ClassBuff[i]);
		printf("%f \n", ResultBuff[i]);
	}
	//=============================近似熵计算测试===================================
	printf("========================Start NICE Apen Test!========================\n");
	begin_instret =  __get_rv_instret();
	begin_cycle   =  __get_rv_cycle();
	//test
	ApEn_Calculate(0.2, Apenbuff, 1);												//计算近似熵
	//end test
	end_instret = __get_rv_instret();
	end_cycle   = __get_rv_cycle();
	instret_nice = end_instret - begin_instret;
	cycle_nice = end_cycle - begin_cycle;
	printf("#nice hardware: \n");
	printf("#instret: %d, cycle: %d \n", instret_nice, cycle_nice);
	printf("########Parameter########\n");
	printf("APEN_DIMENSION_M = %d\n", APEN_DIMENSION_M);
	printf("APEN_DIMENSION_M1 = %d\n", APEN_DIMENSION_M1);
	printf("APEN_MD_LEN = %d\n", APEN_MD_LEN);
	printf("APEN_M1D_LEN = %d\n", APEN_M1D_LEN);
	printf("ApEn_StdVal = %f\n",ApEn_StdVal);
	printf("ApEn_Threshold = %f\n",ApEn_Threshold);
	printf("DistLnAVR_m = %f\n",ApEn_DistLnAVR_m);
	printf("DistLnAVR_m1 = %f\n",ApEn_DistLnAVR_m1);
	printf("ApEn_Result = %f\n",ApEn_Result);

	//使用纯软件计算
	printf("=======================Start Soft Apen Test!=========================\n");
	begin_instret =  __get_rv_instret();
	begin_cycle   =  __get_rv_cycle();
	//test
	ApEn_Calculate(0.2, Apenbuff, 0);												//计算近似熵
	//end test
	end_instret = __get_rv_instret();
	end_cycle   = __get_rv_cycle();
	instret_normal = end_instret - begin_instret;
	cycle_normal = end_cycle - begin_cycle;
	printf("#normal software: \n");
	printf("#instret: %d, cycle: %d \n", instret_normal, cycle_normal);
	printf("########Parameter########\n");
	printf("APEN_DIMENSION_M = %d\n", APEN_DIMENSION_M);
	printf("APEN_DIMENSION_M1 = %d\n", APEN_DIMENSION_M1);
	printf("APEN_MD_LEN = %d\n", APEN_MD_LEN);
	printf("APEN_M1D_LEN = %d\n", APEN_M1D_LEN);
	printf("ApEn_StdVal = %f\n",ApEn_StdVal);
	printf("ApEn_Threshold = %f\n",ApEn_Threshold);
	printf("DistLnAVR_m = %f\n",ApEn_DistLnAVR_m);
	printf("DistLnAVR_m1 = %f\n",ApEn_DistLnAVR_m1);
	printf("ApEn_Result = %f\n",ApEn_Result);
	printf("=================================END=================================\n");
	//测试结束，下面的注释内容是近似熵的计算过程数据
	/*
	printf("===========================ApEn_MD_Buff===========================\n");
	//输出重构的数组
	for(i = 0; i < APEN_MD_LEN; ++i){
		for(j = 0; j < APEN_DIMENSION_M; ++j){
			printf("%.2d ", ApEn_MD_Buff[i][j]);
		}
		printf("\n");
	}
	printf("===========================ApEn_M1D_Buff===========================\n");
	for(i = 0; i < APEN_M1D_LEN; ++i){
		for(j = 0; j < APEN_DIMENSION_M1; ++j){
			printf("%.2d ", ApEn_M1D_Buff[i][j]);
		}
		printf("\n");
	}
	printf("===========================ApEn_DsitanceAll_m===========================\n");
	for(i = 0; i < APEN_MD_LEN; ++i){
		printf("ApEn_DsitanceAll_m[%d] = %d\n", i, ApEn_DsitanceAll_m[i]);
	}
	printf("===========================ApEn_DsitanceAll_m1===========================\n");
	for(i = 0; i < APEN_M1D_LEN; ++i){
		printf("ApEn_DsitanceAll_m1[%d] = %d\n", i, ApEn_DsitanceAll_m1[i]);
	}*/
	//开启对应引脚IOF功能
	//gpio_iof_config(GPIOB, SOC_UART_0_TX_MASK|SOC_UART_0_RX_MASK);
	//初始化串口
	//uart_init(UART1,115200);

	unsigned int revpos = 0;
	unsigned char revbuff[505];
	while(1){
		//读取串口
		revbuff[revpos++] = uart_read(UART0);
		//同步帧头
		if(revpos == 1 && revbuff[0] != 0xaa){
			revpos = 0;
		}
		else if(revpos == 2 && revbuff[1] != 0x55){
			revpos = 0;
		}
		else if(revpos >= 505){
			//打印接收到的数据
			printf("Receive a data package!\n");
			for(i = 0; i < 505; ++i){
				printf("%x,", revbuff[i]);
			}
			printf("\n");
			//将数据还原成浮点
			printf("Reduction to float.\n");
			for(i = 0; i < 250; ++i){
				OriDataBuff[i] = (float)((unsigned int)revbuff[i*2 + 4] + ((unsigned int)revbuff[i*2 + 3] << 8)) * 0.001f - 32.767f;
				printf("%.3f, ", OriDataBuff[i]);
				Apenbuff[i] = (int)revbuff[i] - 32767;
			}
			printf("\n");
			revpos = 0;
			//运行一次神经网络
			# ifdef CNN_NICE_USE
				for(i = 0; i < 6; ++i){
					ClassBuff[i] = 0;
				}
				printf("#Start NICE CNN Analysis!========================\n");
				begin_instret =  __get_rv_instret();
				begin_cycle   =  __get_rv_cycle();
				//test
				//Init_CNN();																	//初始化CNN
				LoadOriData_Calu(1);														//计算CNN
				//end test
				end_instret = __get_rv_instret();
				end_cycle   = __get_rv_cycle();
				instret_nice = end_instret - begin_instret;
				cycle_nice = end_cycle - begin_cycle;
				printf("#instret: %d, cycle: %d \n", instret_nice, cycle_nice);
				//输出分类结果
				printf("@Result:\n");
				for(i = 0; i < 6; ++i){
					printf("class %d : %d ", i, ClassBuff[i]);
					printf("%f \n", ResultBuff[i]);
				}
			# endif
				for(i = 0; i < 6; ++i){
					ClassBuff[i] = 0;
				}
				printf("#Start Soft CNN Analysis!========================\n");
				begin_instret =  __get_rv_instret();
				begin_cycle   =  __get_rv_cycle();
				//test
				//Init_CNN();																//初始化CNN
				LoadOriData_Calu(0);														//计算CNN
				//end test
				end_instret = __get_rv_instret();
				end_cycle   = __get_rv_cycle();
				instret_nice = end_instret - begin_instret;
				cycle_nice = end_cycle - begin_cycle;
				printf("#instret: %d, cycle: %d \n", instret_nice, cycle_nice);
				//输出分类结果
				printf("@Result:\n");
				for(i = 0; i < 6; ++i){
					printf("class %d : %d ", i, ClassBuff[i]);
					printf("%f \n", ResultBuff[i]);
				}
				printf("#Analysis end!==================================\n");
				printf("#Start Nice Apen Analysis!========================\n");
				begin_instret =  __get_rv_instret();
				begin_cycle   =  __get_rv_cycle();
				//test
				ApEn_Calculate(0.2, Apenbuff, 1);											//计算近似熵
				//end test
				end_instret = __get_rv_instret();
				end_cycle   = __get_rv_cycle();
				instret_nice = end_instret - begin_instret;
				cycle_nice = end_cycle - begin_cycle;
				printf("#instret: %d, cycle: %d \n", instret_nice, cycle_nice);
				printf("APEN_DIMENSION_M = %d\n", APEN_DIMENSION_M);
				printf("APEN_DIMENSION_M1 = %d\n", APEN_DIMENSION_M1);
				printf("APEN_MD_LEN = %d\n", APEN_MD_LEN);
				printf("APEN_M1D_LEN = %d\n", APEN_M1D_LEN);
				printf("ApEn_StdVal = %f\n",ApEn_StdVal);
				printf("ApEn_Threshold = %f\n",ApEn_Threshold);
				printf("DistLnAVR_m = %f\n",ApEn_DistLnAVR_m);
				printf("DistLnAVR_m1 = %f\n",ApEn_DistLnAVR_m1);
				printf("ApEn_Result = %f\n",ApEn_Result);
				//通过Apen纠正CNN的误判
				if(ApEn_Result < 0.85 && ClassBuff[5] > 0){
					printf("*Correction of CNN misjudgment by approximate entropy algorithm!\n");
				}

				printf("#Analysis end!==================================\n");
		}
	}
	return 0;
}

//初始化CNN,配置CNN程序的参数，通过配置这个函数可以建立网络关系
void Init_CNN(void)
{
	//=========================一共有1层 ========================
	cnninst1.LayerAmount = 1;

	//========================设置每层的卷积核数量========================
	cnninst1.kernelAmount[0] = 1;												//第一层卷积核数量为1个
	//========================设置每层的卷积核长度========================
	cnninst1.kernelLen[0] = 11;													//卷积核长度为9
	//========================设置卷积核buff起始地址========================
	cnninst1.Kernel0BuffStartAddr[0] =  (CALU_TYPE*)kernel0; 					//第一层的卷积核的存放地址
	//========================设置池化模式========================

	//========================设置每层的预padding数量========================
	cnninst1.PaddingAmount[0] = 0;												//除第一层外，其它层的padding是为下一层计算进行的预padding，
	//========================设置原始序列的地址========================
	cnninst1.pOriginalData = OriDataBuff;
	cnninst1.OriginalDatalen = 250;
}

float allnetlayer1[128];
float allnetlayer2[32];
//卷积神经网络计算实现函数
void LoadOriData_Calu(int usenice)
{
	int i = 0, j = 0, maxch = 0;;
	float tempvalue;
	//计算CNN
	if(usenice){
		# ifdef CNN_NICE_USE
		CNN_RunAllLayer_inst_Nice(&cnninst1);
		# endif
	}
	else{
		CNN_RunAllLayer_inst(&cnninst1);
	}
	if(usenice){
		//计算全连接层
		for(i = 0; i < 6; ++i){
			//Relu 函数
			CNN_Calurelu(ConvCache1, 992);
			//计算全连接层的结果
			# ifdef CNN_NICE_USE
			//ResultBuff[i] = CNN_Calculate_Full_Connection_Nice(ConvCache1, (CALU_TYPE*)(&FullNetPara[i][0]), 992);
			//计算全连接层1
			CNN_Calurelu(ConvCache1, 80);
			for(i = 0; i < 128; ++i){
				allnetlayer1[i] = CNN_Calculate_Full_Connection_Nice(ConvCache1, (CALU_TYPE*)(&FullNetPara0[i][0]), 80);
			}
			//计算全连接层2
			CNN_Calurelu(allnetlayer1, 128);
			for(i = 0; i < 32; ++i){
				allnetlayer2[i] = CNN_Calculate_Full_Connection_Nice(allnetlayer1, (CALU_TYPE*)(&FullNetPara1[i][0]), 128);
			}
			//计算全连接层3
			CNN_Calurelu(allnetlayer2, 32);
			for(i = 0; i < 6; ++i){
				ResultBuff[i] = CNN_Calculate_Full_Connection_Nice(allnetlayer2, (CALU_TYPE*)(&FullNetPara2[i][0]), 32);
			}
			# endif
		}
	}
	else{
		//计算全连接层1
		CNN_Calurelu(ConvCache1, 80);
		for(i = 0; i < 128; ++i){
			allnetlayer1[i] = CNN_Calculate_Full_Connection(ConvCache1, (CALU_TYPE*)(&FullNetPara0[i][0]), 80);
		}
		//计算全连接层2
		CNN_Calurelu(allnetlayer1, 128);
		for(i = 0; i < 32; ++i){
			allnetlayer2[i] = CNN_Calculate_Full_Connection(allnetlayer1, (CALU_TYPE*)(&FullNetPara1[i][0]), 128);
		}
		//计算全连接层3
		CNN_Calurelu(allnetlayer2, 32);
		for(i = 0; i < 6; ++i){
			ResultBuff[i] = CNN_Calculate_Full_Connection(allnetlayer2, (CALU_TYPE*)(&FullNetPara2[i][0]), 32);
		}
	}
	//获取最大值
	tempvalue = ResultBuff[0];
	maxch = 0;
	for(i = 1; i < 6; ++i){
		if(tempvalue < ResultBuff[i]){
			tempvalue = ResultBuff[i];
			maxch = i;
		}
	}
	//由最大值得到分类
	ClassBuff[maxch] += 1;
}

//卷积向量计算函数
float MyConv_CaluFunction(float* buff1, float* buff2, int len)
{
	int i = 0;
	float sum = 0;
	//进行一次向量乘法
	for(i = 0; i < len; ++i){
		sum += 	buff1[i] * buff2[i];
	}
	return sum;
}
