#ifndef __INSN_H__
#define __INSN_H__

#include <hbird_sdk_soc.h>

#define  ROW_LEN    3
#define  COL_LEN    3

////////////////////////////////////////////////////////////
// custom3:
// Supported format: only R type here
// Supported instr:
//  1. custom3 lbuf: load data(in memory) to row_buf
//     lbuf (a1)
//     .insn r opcode, func3, func7, rd, rs1, rs2    
//  2. custom3 sbuf: store data(in row_buf) to memory
//     sbuf (a1)
//     .insn r opcode, func3, func7, rd, rs1, rs2    
//  3. custom3 acc rowsum: load data from memory(@a1), accumulate row datas and write back 
//     rowsum rd, a1, x0
//     .insn r opcode, func3, func7, rd, rs1, rs2    
////////////////////////////////////////////////////////////

// custom lbuf 
__STATIC_FORCEINLINE int custom_lbuf(unsigned int addr)
{
    int rt = 0;
    
    asm volatile (
       ".insn r 0x7b, 6, 1, %0, %1, x0"
           :"=r"(rt)
           :"r"(addr)
     );

    return rt;
}

// custom lth
__STATIC_FORCEINLINE int custom_lth(int threshold)
{
    int rt = 0;
    
    asm volatile (
       ".insn r 0x7b, 6, 2, %0, %1, x0"
           :"=r"(rt)
           :"r"(threshold)
     );

    return rt;
}

// custom calu_apen
__STATIC_FORCEINLINE int custom_calu_apen(unsigned int Addr)
{
    int rt = 0;

    asm volatile (
       ".insn r 0x7b, 6, 3, %0, %1, x0"
           :"=r"(rt)
           :"r"(Addr)
     );
    return rt;
}


// custom load cnn kerenl
__STATIC_FORCEINLINE int custom_cnn_load_kernel(unsigned int Addr, int Len)
{
    int rt = 0;

    asm volatile (
       ".insn r 0x7b, 7, 4, %0, %1, %2"
           :"=r"(rt)
           :"r"(Addr),"r"(Len)
     );

    return rt;
}


// custom calculate convolution
__STATIC_FORCEINLINE float custom_cnn_calculate(unsigned int Addr)
{
    float rt = 0;
    
    asm volatile (
       ".insn r 0x7b, 6, 5, %0, %1, x0"
           :"=r"(rt)
           :"r"(Addr)
     );
    
    return rt;
}

// multiply
__STATIC_FORCEINLINE int custom_multiply(int a, int b)
{
    int rt = 0;

    asm volatile (
       ".insn r 0x7b, 7, 6, %0, %1, %2"
           :"=r"(rt)
           :"r"(a),"r"(b)
     );

    return rt;
}

// multiply float
__STATIC_FORCEINLINE float custom_multiply_f(float a, float b)
{
    float rt = 0;

    asm volatile (
       ".insn r 0x7b, 7, 7, %0, %1, %2"
           :"=r"(rt)
           :"r"(a),"r"(b)
     );

    return rt;
}

#endif

